"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTransport = void 0;
const build = require("pino-abstract-transport");
const https = require("https");
const node_fetch_1 = require("node-fetch");
const DEFAULT_COLORS = { 30: '#2EB67D', 40: '#ECB22E', 50: '#E01E5A', 60: '#E01E5A' };
const DEFAULT_EXCLUDED_KEYS = { hostname: 0, pid: 0 };
const DEFAULT_MESSAGE_KEY = 'msg';
const flatten = (obj, keys = []) => {
    return Object.entries(obj).reduce((acc, [key, value]) => (Object.assign(Object.assign({}, acc), value !== null && typeof value === 'object' ? flatten(value, keys.concat(key)) : { [keys.concat(key).join('.')]: String(value) })), {});
};
const process = ({ agent, log, options: { channelKey, colors = DEFAULT_COLORS, excludedKeys = DEFAULT_EXCLUDED_KEYS, imageUrlKey, messageKey = DEFAULT_MESSAGE_KEY, webhookUrl } }) => __awaiter(void 0, void 0, void 0, function* () {
    const _a = log, _b = messageKey, msg = _a[_b], { time, level } = _a, bindings = __rest(_a, [typeof _b === "symbol" ? _b : _b + "", "time", "level"]);
    const payload = {
        blocks: []
    };
    if (typeof channelKey === 'string') {
        payload.channel = bindings[channelKey];
    }
    if (typeof msg === 'string') {
        payload.text = msg;
        payload.blocks.push({
            type: 'section',
            text: { type: 'mrkdwn', text: `\`\`\`${msg}\`\`\`` }
        });
    }
    const date = new Date(time);
    payload.blocks.push({
        type: 'context',
        elements: [{ type: 'mrkdwn', text: `<!date^${Math.floor(date.getTime() / 1000)}^Posted {date_pretty} at {time_secs}|Posted ${date.toString()}>` }]
    });
    const data = Object.keys(bindings).reduce((acc, key) => (Object.assign(Object.assign({}, acc), key in excludedKeys ? {} : { [key]: bindings[key] })), {});
    const fields = Object.entries(flatten(data)).map(([title, value]) => ({
        title,
        value,
        short: value.length < 500
    }));
    const imageUrl = typeof imageUrlKey === 'string' ? bindings[imageUrlKey] : undefined;
    if (fields.length > 0 || typeof imageUrl === 'string') {
        payload.attachments = [{
                color: colors[level],
                fields,
                image_url: imageUrl
            }];
    }
    yield (0, node_fetch_1.default)(webhookUrl, {
        agent,
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify(payload)
    });
});
const createTransport = (options) => {
    const agent = options.keepAlive === true
        ? new https.Agent({ keepAlive: true })
        : undefined;
    return build((iterable) => { var iterable_1, iterable_1_1; return __awaiter(void 0, void 0, void 0, function* () {
        var e_1, _a;
        try {
            for (iterable_1 = __asyncValues(iterable); iterable_1_1 = yield iterable_1.next(), !iterable_1_1.done;) {
                const log = iterable_1_1.value;
                try {
                    yield process({
                        agent,
                        log,
                        options
                    });
                }
                catch (err) {
                    console.error(err);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (iterable_1_1 && !iterable_1_1.done && (_a = iterable_1.return)) yield _a.call(iterable_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }); });
};
exports.createTransport = createTransport;
exports.default = exports.createTransport;
